unit AgeEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TAgeEdit = class(TCustomEdit)
  private
    fValue, fYear,fMonth: Integer;
    fAgeFormat: string;
    function GetAgeValue: Integer;
    procedure SetAgeValue(const Value: Integer);
    procedure SetText(pYear,pMonth: Integer);
    procedure WMKeyDown(var Message: TWMKeyDown);message WM_KEYDOWN;
    procedure WMChar(var Message: TWMChar);message WM_CHAR;
    procedure CMEnter(var Message: TCMEnter);message CM_ENTER;
    procedure HighlightToken(pText: string);
    procedure GetValues;
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent);override;
  published
    { Published declarations }
    property Value: Integer read GetAgeValue write SetAgeValue;
    property Anchors;
    property AutoSelect;
    property AutoSize;
    property BiDiMode;
    property BorderStyle;
    property CharCase;
    property Color;
    property Constraints;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property ImeMode;
    property ImeName;
    property MaxLength;
    property OEMConvert;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation
{$R *.res}
procedure Register;
begin
  RegisterComponents('Age Suit', [TAgeEdit]);
end;

{ TAgeEdit }

procedure TAgeEdit.CMEnter(var Message: TCMEnter);
begin
        SendMessage(Handle, EM_SETSEL, 0, 2);
end;

constructor TAgeEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
        fValue := 0;
        fYear:= 0;
        fMonth := 0;
        fAgeFormat := '00 year(s) 00 month(s)';
end;

function TAgeEdit.GetAgeValue: Integer;
begin
        Result := fValue;
end;

procedure TAgeEdit.GetValues;
begin
        fYear := StrToInt(Copy(Self.Text,1,2));
        fMonth := StrToInt(Copy(Self.Text,12,2));
        if fMonth > 11 then begin
                fMonth := 11;
                SetText(fYear,fMonth);
        end;
        fValue := fYear*12+fMonth;
end;

procedure TAgeEdit.HighlightToken(pText: string);
var startpos,endpos,i: Integer;
begin

end;

procedure TAgeEdit.SetAgeValue(const Value: Integer);
begin
        fValue := Value;
        fYear := Value div 12;
        fMonth := Value mod 12;
        SetText(fYear,fMonth);
end;

procedure TAgeEdit.SetText(pYear, pMonth: Integer);
begin
        Self.Text := FormatFloat('00 year(s) ',pYear)+FormatFloat('00 month(s)',pMonth);
end;

procedure TAgeEdit.WMChar(var Message: TWMChar);
var lStart: Integer;
Y1,M1: Char;
s: string;
begin
        s := Self.Text;
        Y1 := s[1];
        M1 := s[12];
        lStart := SelStart;
        if lStart = 2 then lStart := 11;
        SelStart := lStart;
        if (SelStart = 0) and (Char(Message.CharCode) in ['0'..'9']) then Self.Text := Char(Message.CharCode)+'0 year(s) '+FormatFloat('00 month(s)',fMonth);
        if (SelStart = 1) and (Char(Message.CharCode) in ['0'..'9']) then Self.Text := Y1+Char(Message.CharCode)+' year(s) '+FormatFloat('00 month(s)',fMonth);
        if (SelStart = 11) and (Char(Message.CharCode) in ['0'..'1']) then Self.Text := FormatFloat('00 year(s) ',fYear)+Char(Message.CharCode)+'0 month(s)';
        if (SelStart = 12) and (Char(Message.CharCode) in ['0'..'9']) then Self.Text := FormatFloat('00 year(s) ',fYear)+M1+Char(Message.CharCode)+' month(s)';
        GetValues;
        Inc(lStart);
        SelStart := lStart;
end;

procedure TAgeEdit.WMKeyDown(var Message: TWMKeyDown);
var Year: boolean;
begin
        Year := (SelStart < 11);
        if Message.CharCode=VK_UP then begin
                if Year then Inc(fYear)
                else Inc(fMonth);
                if fYear>99 then fYear:=99;
                if fMonth > 11 then begin
                        fMonth := 0;
                        Inc(fYear);
                end;
                SetText(fYear,fMonth);
                fValue := fYear*12+fMonth;
                if Year then   SendMessage(Handle, EM_SETSEL, 0, 2)
                else SendMessage(Handle, EM_SETSEL, 11, 13);
                exit;
        end;
        if Message.CharCode=VK_DOWN then begin
                if Year then Dec(fYear)
                else Dec(fMonth);
                if fYear<0 then fYear := 0;
                if fMonth < 0 then begin
                        fMonth := 11;
                        Dec(fYear);
                        if fYear<0 then begin
                                fYear := 0;
                                fMonth := 0;
                        end;
                end;
                SetText(fYear,fMonth);
                fValue := fYear*12+fMonth;
                if Year then   SendMessage(Handle, EM_SETSEL, 0, 2)
                else SendMessage(Handle, EM_SETSEL, 11, 13);
                exit;
        end;
        if Message.CharCode = VK_RIGHT then begin
                SendMessage(Handle,EM_SETSEL,11,13);
                exit;
        end;
        if Message.CharCode = VK_LEFT then begin
                SendMessage(Handle, EM_SETSEL, 0, 2);
                exit;
        end;
        if Message.CharCode=VK_DELETE then begin
                if Year then
                        fYear := 0
                else    fMonth := 0;
                fValue := fYear*12+fMonth;
                SetText(fYear,fMonth);
                if Year then
                SendMessage(Handle, EM_SETSEL, 0, 2)
                else SendMessage(Handle, EM_SETSEL, 11, 13);
                exit;
        end;
inherited;
end;

end.
