unit DBAgeEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, DBCtrls, DB;

type
  TDBAgeEdit = class(TCustomEdit)
  private
    { Private declarations }
    FDataLink:  TFieldDataLink;
    fValue, fYear,fMonth: Integer;
    fAgeFormat: string;
    FFocused: boolean;
    procedure ActiveChange(Sender: TObject);
    procedure DataChange(Sender: TObject);
    procedure EditingChange(Sender: TObject);
    procedure UpdateData(Sender: TObject);
    function GetAgeValue: Integer;
    procedure SetAgeValue(const Value: Integer);
    procedure SetText(pYear,pMonth: Integer);
    procedure WMKeyDown(var Message: TWMKeyDown);message WM_KEYDOWN;
    procedure WMChar(var Message: TWMChar);message WM_CHAR;
    procedure CMEnter(var Message: TCMEnter);message CM_ENTER;
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;    
    procedure GetValues;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(const Value: TDataSource);
    function GetField: TField;
    procedure SelYear;
    procedure SelMonth;
    procedure SetFocused(Value: boolean);
  protected
    { Protected declarations }
    procedure Change; override;
    function EditCanModify: Boolean;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure Reset;    
  public
    { Public declarations }
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
    property Value: Integer read GetAgeValue write SetAgeValue;
    property Field: TField read GetField;        
  published
    { Published declarations }
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;    
    property Anchors;
    property AutoSelect;
    property AutoSize;
    property BiDiMode;
    property BorderStyle;
    property CharCase;
    property Color;
    property Constraints;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property ImeMode;
    property ImeName;
    property MaxLength;
    property OEMConvert;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation
{$R *.res}
procedure Register;
begin
  RegisterComponents('Age Suit', [TDBAgeEdit]);
end;

{ TDBAgeEdit }

procedure TDBAgeEdit.ActiveChange(Sender: TObject);
begin
        if FDataLink.DataSet.Active then
        Value := fDataLink.Field.AsInteger
        else Value := 0;
end;

procedure TDBAgeEdit.Change;
begin
  FDataLink.Modified;
  inherited Change;
end;

procedure TDBAgeEdit.CMEnter(var Message: TCMEnter);
begin
 inherited;
        SetFocused(true);
        SelYear;
end;

procedure TDBAgeEdit.CMExit(var Message: TCMExit);
begin
  try
    FDataLink.UpdateRecord;
  except
    SelectAll;
    SetFocus;
    raise;
  end;
  SetFocused(False);
end;

procedure TDBAgeEdit.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

constructor TDBAgeEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnEditingChange := EditingChange;
  FDataLink.OnUpdateData := UpdateData;
  FDataLink.OnActiveChange := ActiveChange;
  fAgeFormat := '00 year(s) 00 month(s)';
  Text := fAgeFormat;
end;

procedure TDBAgeEdit.DataChange(Sender: TObject);
begin
    if FDataLink.Field <> nil then
    if FFocused and FDataLink.CanModify then
       if FDataLink.DataSet.Active then
       Value := FDataLink.Field.AsInteger
       else Value := 0
    else
    begin
      if FDataLink.DataSet.Active then
      Value := FDataLink.Field.AsInteger
      else Value := 0;
      if FDataLink.Editing then
        Modified := True;
    end;
end;

destructor TDBAgeEdit.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;

end;

function TDBAgeEdit.EditCanModify: Boolean;
begin
  Result := FDataLink.Edit;
end;

procedure TDBAgeEdit.EditingChange(Sender: TObject);
begin
  inherited ReadOnly := not FDataLink.Editing;
end;

function TDBAgeEdit.GetAgeValue: Integer;
begin
        Result := fValue;
end;

function TDBAgeEdit.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

function TDBAgeEdit.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

function TDBAgeEdit.GetField: TField;
begin
        Result := FDataLink.Field;
end;

procedure TDBAgeEdit.GetValues;
begin
        fYear := StrToInt(Copy(Self.Text,1,2));
        fMonth := StrToInt(Copy(Self.Text,12,2));
        if fMonth > 11 then begin
                fMonth := 11;
                SetText(fYear,fMonth);
        end;
        fValue := fYear*12+fMonth;
end;


procedure TDBAgeEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if (Key = VK_DELETE) or ((Key = VK_INSERT) and (ssShift in Shift)) then
    FDataLink.Edit;
end;

procedure TDBAgeEdit.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if Key in ['0'..'9'] then FDataLink.Edit;
  if Key = #27 then begin
        FDataLink.Reset;
        Key := #0;
  end;
end;

procedure TDBAgeEdit.Loaded;
begin
  inherited Loaded;
  if (csDesigning in ComponentState) then DataChange(Self);
end;

procedure TDBAgeEdit.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TDBAgeEdit.Reset;
begin
        FDataLink.Reset;
        SelYear;        
end;

procedure TDBAgeEdit.SelMonth;
begin
        SendMessage(Handle, EM_SETSEL, 11, 13);
end;

procedure TDBAgeEdit.SelYear;
begin
        SendMessage(Handle, EM_SETSEL, 0, 2);
end;

procedure TDBAgeEdit.SetAgeValue(const Value: Integer);
begin
        if FDataLink.DataSet.Active then begin
        fValue := Value;
        fYear := Value div 12;
        fMonth := Value mod 12;
        end
        else begin
                fValue := 0;
                fYear := 0;
                fMonth := 0;
        end;
        SetText(fYear,fMonth);
end;

procedure TDBAgeEdit.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

procedure TDBAgeEdit.SetDataSource(const Value: TDataSource);
begin
  if not (FDataLink.DataSourceFixed and (csLoading in ComponentState)) then
    FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

procedure TDBAgeEdit.SetFocused(Value: boolean);
begin
        if FFocused <> Value then begin
                FFocused := Value;
                FDataLink.Reset;
        end;
end;

procedure TDBAgeEdit.SetText(pYear, pMonth: Integer);
begin
        Self.Text := FormatFloat('00 year(s) ',pYear)+FormatFloat('00 month(s)',pMonth);
end;

procedure TDBAgeEdit.UpdateData(Sender: TObject);
begin
  FDataLink.Field.AsInteger := fValue;
end;

procedure TDBAgeEdit.WMChar(var Message: TWMChar);
var lStart: Integer;
Y1,M1: Char;
s: string;
begin
        s := Self.Text;
        Y1 := s[1];
        M1 := s[12];
        lStart := SelStart;
        if lStart = 2 then lStart := 11;
        SelStart := lStart;
        if (SelStart = 0) and (Char(Message.CharCode) in ['0'..'9']) then Self.Text := Char(Message.CharCode)+'0 year(s) '+FormatFloat('00 month(s)',fMonth);
        if (SelStart = 1) and (Char(Message.CharCode) in ['0'..'9']) then Self.Text := Y1+Char(Message.CharCode)+' year(s) '+FormatFloat('00 month(s)',fMonth);
        if (SelStart = 11) and (Char(Message.CharCode) in ['0'..'1']) then Self.Text := FormatFloat('00 year(s) ',fYear)+Char(Message.CharCode)+'0 month(s)';
        if (SelStart = 12) and (Char(Message.CharCode) in ['0'..'9']) then Self.Text := FormatFloat('00 year(s) ',fYear)+M1+Char(Message.CharCode)+' month(s)';
        GetValues;
        Inc(lStart);
        SelStart := lStart;
        if (Char(Message.CharCode) in ['0'..'9']){ and (not FDataLink.Editing)} then begin
                FDataLink.Edit;
                FDataLink.Field.Value := fValue;
        end;
        if Char(Message.CharCode) = #27 then begin
              FDataLink.Reset;
              Message.CharCode := 0;
        end;
end;

procedure TDBAgeEdit.WMKeyDown(var Message: TWMKeyDown);
var Year: boolean;
begin
        Year := (SelStart < 11);
        if Message.CharCode=VK_UP then begin
                if Year then Inc(fYear)
                else Inc(fMonth);
                if fYear>99 then fYear:=99;
                if fMonth > 11 then begin
                        fMonth := 0;
                        Inc(fYear);
                end;
                SetText(fYear,fMonth);
                fValue := fYear*12+fMonth;
                if Year then   SelYear
                else SelMonth;
                if not FDataLink.Editing then
                FDataLink.Edit;
                FDataLink.Field.Value := fValue;
                exit;
        end;
        if Message.CharCode=VK_DOWN then begin
                if Year then Dec(fYear)
                else Dec(fMonth);
                if fYear<0 then fYear := 0;
                if fMonth < 0 then begin
                        fMonth := 11;
                        Dec(fYear);
                        if fYear<0 then begin
                                fYear := 0;
                                fMonth := 0;
                        end;
                end;
                SetText(fYear,fMonth);
                fValue := fYear*12+fMonth;
                if Year then   SelYear
                else SelMonth;
                if not FDataLink.Editing then
                FDataLink.Edit;
                FDataLink.Field.Value := fValue;
                exit;
        end;
        if Message.CharCode = VK_RIGHT then begin
                SelMonth;
                exit;
        end;
        if Message.CharCode = VK_LEFT then begin
                SelYear;
                exit;
        end;
        if Message.CharCode=VK_DELETE then begin
                if Year then
                        fYear := 0
                else    fMonth := 0;
                fValue := fYear*12+fMonth;
                SetText(fYear,fMonth);
                if Year then
                SelYear
                else SelMonth;
                if not FDataLink.Editing then
                FDataLink.Edit;
                FDataLink.Field.Value := fValue;                
                exit;
        end;
inherited;
end;

end.
